#include <stdio.h>
#include <string.h>
#include <iostream>
#include "gcm.h"

#define false 0
#define true 1

using namespace std;

////////////////////////////////////////////////////////////////////////////
//Constructor();
////////////////////////////////////////////////////////////////////////////
gcm::gcm(char* name){

	isopen=false;
	filename=name;	
	memset(wipebuffer,0,4096);
}

////////////////////////////////////////////////////////////////////////////
//Destructor();
////////////////////////////////////////////////////////////////////////////
gcm::~gcm(){

	Close();
}

////////////////////////////////////////////////////////////////////////////
//Open();
//Creates a filehandle. We can use fopen here, no largefile support needed.
//It also fills the gcm-iso struct.
//
//In : void
//Out: bool : true=success, false=failure
////////////////////////////////////////////////////////////////////////////
bool gcm::Open(void){
	
	handle=fopen(filename,"rb+");
	if(handle!=0){
		isopen=true;
		
		/* Fill needed values */
		FillValues();
	}
	return(isopen);
}


////////////////////////////////////////////////////////////////////////////
//Close();
//Closes the gcm file.
//
//In : void
//Out: void
////////////////////////////////////////////////////////////////////////////
void gcm::Close(void){

	if(isopen) fclose(handle);
}


////////////////////////////////////////////////////////////////////////////
//FillValues();
//Fills needed membervariables in class.
//
//In : void
//Out: void
////////////////////////////////////////////////////////////////////////////
void gcm::FillValues(void){
	int i;

	/* TOCStart */
    fseek(handle,0x424,SEEK_SET);
	fread(&i,4,1,handle);
	TOCStart=IntSwap(i);

	/* TOCAsciiStart */
	fseek(handle,TOCStart+0x8,SEEK_SET);
	fread(&i,4,1,handle);
	i=IntSwap(i);
	TOCAsciiStart=TOCStart+(i*0xc);

	/* TOCSize */
	TOCSize=TOCAsciiStart-TOCStart;

	/* GCMSize */
	fseek(handle,0,SEEK_END);
	GCMSize=(int)ftell(handle);

}

////////////////////////////////////////////////////////////////////////////
//GetTOCStart();
//Returns TOCStart Offset.
//
//In : void
//Out: int TOCStart
////////////////////////////////////////////////////////////////////////////
int gcm::GetTOCStart(void){

	return(TOCStart);
}


////////////////////////////////////////////////////////////////////////////
//GetTOCAsciiStart();
//Returns TOCAsciiStart Offset.
//
//In : void
//Out: int TOCAsciiStart
////////////////////////////////////////////////////////////////////////////
int gcm::GetTOCAsciiStart(void){

	return(TOCAsciiStart);
}

////////////////////////////////////////////////////////////////////////////
//GetTOCSize();
//Returns TOCSize.
//
//In : void
//Out: int TOCSize
////////////////////////////////////////////////////////////////////////////
int gcm::GetTOCSize(void){

	return(TOCSize);
}


////////////////////////////////////////////////////////////////////////////
//GetGCMSize();
//Returns GCMSize.
//
//In : void
//Out: int GCMSize
////////////////////////////////////////////////////////////////////////////
int gcm::GetGCMSize(void){

	return(GCMSize);
}


////////////////////////////////////////////////////////////////////////////
//GetStream();
//Returns FileStream.
//
//In : void
//Out: FILE* stream
////////////////////////////////////////////////////////////////////////////
FILE* gcm::GetStream(void){

        return(handle);
}

////////////////////////////////////////////////////////////////////////////
//Wipe();
//Wipes a specified area within the ISO with 0x00.
//
//In : int start, int amount
//Out: void
////////////////////////////////////////////////////////////////////////////
void gcm::Wipe(int start, int amount){
	int rest;

	rest=amount;
	amount=amount/4096;
	rest=(rest-amount*4096);

	fseek(handle,start,SEEK_SET);
	for(int i=0;i<amount;i++) fwrite(wipebuffer,4096,1,handle);
	if(rest) fwrite(wipebuffer,rest,1,handle);

}


////////////////////////////////////////////////////////////////////////////
//IntSwap();
//Endianswap for an integer value.
//
//In : int intput_int
//Out: int swapped_int
////////////////////////////////////////////////////////////////////////////
int gcm::IntSwap(int x){

    return(
            ((x >> 24) & 0x000000ff) |
            ((x >> 8)  & 0x0000ff00) |
            ((x << 8)  & 0x00ff0000) |
            ((x << 24) & 0xff000000)
          );
}

