#include <stdio.h>
#include <iostream>
#include <iomanip>
#include <vector>
#include "gcm.h"


using namespace std;
int	IntSwap(int x); // Small prototype for IntegerSwap
typedef	vector<int> IntegerVector; // We need a vector which stores integer values
struct fststruct{ short dir; short dummy; int start; int size; } fst; // GC filesystem stuct


int main(int argc, char **argv){
	int z=0, info=0, i, tocstart, tocsize, gcmsize, count, requested, found, percent=0;

	/* Print the versionstring to stdout */
	cout << "GCMWipe Version 0.7.3 (mperry@equinox.com)" << endl;

	/* We don't have enough arguments or no are arguments given. */
	if(argc!=2){
		cout << "Wipes garbage bytes within GCM images" << endl << "Usage: GCMWipe <gcm-file>" << endl;
		return(-1);
	}

	/* Open the GCM image */
	gcm *gcmiso = new gcm(argv[1]);

	
	if(!gcmiso->Open()){
		cout << "Error: could not open file " << argv[1] << endl;
		delete(gcmiso);
		return(-1);
	}


	/* We need some vectors and iterators to store every filestart and filelength in gcmiso */
	IntegerVector SizeVector;
	IntegerVector StartVector;
	IntegerVector::iterator SizeVectorIterator;
	IntegerVector::iterator StartVectorIterator;
	IntegerVector::iterator StartVectorIterator2;

	/* Now get the right values from the gcmiso */
	tocstart=gcmiso->GetTOCStart();
	tocsize=gcmiso->GetTOCSize();
	gcmsize=gcmiso->GetGCMSize();


	/* We start at TOCStart, so fseek to TOCStart */
	fseek(gcmiso->GetStream(),gcmiso->GetTOCStart(),SEEK_SET);

	/* This loop collects all filestarts and filelengths in two different vectorobjects */
	do{
		fread(&fst,12,1,gcmiso->GetStream());
		fst.size=IntSwap(fst.size);
		fst.start=IntSwap(fst.start);
		if(fst.dir==0){
			SizeVector.push_back(fst.size);
			StartVector.push_back(fst.start);
		}
		tocsize-=12;
	}while(tocsize!=0);

	/* Get the size of one of the two vectors. Doesn't matter which one, both have the same size. */
	count=SizeVector.size();
	
	/*******************************************************************************************
	  Okay, this the main algorithm. It does the following:

      1. Takes two entries of each vector and adds them. Then we know that file[i] ends at offset x

	  2. Now we need to find out which file entry is the next to offset x of above file.

      3. Wipe the gap between them because it contains crap. (found-requested)

      This algorithm needs alot of time and is not optimized at all.
	  What can *you* do against this (because I am too lazy)?

	  - Sort the values within the vector
	  - Use divide and conquer method
	
	 ******************************************************************************************/

	SizeVectorIterator = SizeVector.begin();
	StartVectorIterator = StartVector.begin();

	cout << "Wiping garbage between files, this could take a while..."; fflush(stdout);
	do{
		/* requested = where does this file end ? */
		requested=*StartVectorIterator+*SizeVectorIterator;
		StartVectorIterator++;
		SizeVectorIterator++;

		/* set found initially to gcmsize */
		found=gcmsize;

		/* Go through the StartVector to check for next file-start close to requested */
		StartVectorIterator2 = StartVector.begin();
		for(i=0; i< count; i++){
			z=*StartVectorIterator2;
			StartVectorIterator2++;
			if(z>=requested){
				if(z<found) found=z;
			}
		}

		/* Security: we do this if-check so we do wipe negative values! */
		if((found-requested)!=0){
			/* Wipe it ! */
			gcmiso->Wipe(requested, found-requested);
			/* Log wiped bytes for our information */
			info+=(found-requested);
		}

	}while(SizeVectorIterator != SizeVector.end());

	cout << " done." << endl;
	cout << "Wiped: " << info << " bytes (approx. " << (info/1024000) << "MB)." << endl;


	/* Over and out ! */
	SizeVector.clear();
	StartVector.clear();
	delete(gcmiso);
	return(0);
}

int IntSwap(int x){

	return( ((x >> 24) & 0x000000ff) | ((x >> 8)  & 0x0000ff00) | ((x << 8)  & 0x00ff0000) | ((x << 24) & 0xff000000) );
}

